# Generated by Django 3.1 on 2020-08-05 02:39

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("polls", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="RelatedData",
            fields=[
                (
                    "id",
                    models.CharField(max_length=32, primary_key=True, serialize=False),
                ),
                ("extra_data", models.TextField(blank=True, default="")),
            ],
        ),
        migrations.AlterField(
            model_name="comment",
            name="uuid",
            field=models.UUIDField(
                default=uuid.uuid4, editable=False, primary_key=True, serialize=False
            ),
        ),
        migrations.AlterField(
            model_name="poll",
            name="pub_date",
            field=models.DateTimeField(verbose_name="date published"),
        ),
    ]



import datetime
from uuid import uuid4

from django.db import models
from django.utils import timezone


class Poll(models.Model):
    question = models.CharField(max_length=200)
    pub_date = models.DateTimeField("date published")

    def __str__(self):
        return self.question

    def was_published_recently(self):
        return self.pub_date >= timezone.now() - datetime.timedelta(days=1)

    was_published_recently.admin_order_field = "pub_date"
    was_published_recently.boolean = True
    was_published_recently.short_description = "Published recently?"


class Choice(models.Model):
    poll = models.ForeignKey(Poll, on_delete=models.CASCADE)
    choice_text = models.CharField(max_length=200)
    votes = models.IntegerField()

    def __str__(self):
        return self.choice_text


class Comment(models.Model):
    uuid = models.UUIDField(primary_key=True, default=uuid4, editable=False)
    comment = models.TextField(null=True, blank=True)

    def __str__(self):
        return self.comment or ""


class RelatedData(models.Model):
    id = models.CharField(primary_key=True, max_length=32)
    extra_data = models.TextField(blank=True, default="")

    def __str__(self):
        return self.extra_data or self.id



# Django settings for example_project project.
import os

import dj_database_url


def project_dir(*paths):
    base = os.path.realpath(os.path.dirname(__file__))
    return os.path.join(base, *paths)


DEBUG = True

DATABASES = {
    "default": dj_database_url.config(
        default="sqlite:///" + project_dir("example_project.db")
    )
}
DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = "America/Chicago"

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = "en-us"

SITE_ID = 1

USE_L10N = False

USE_TZ = False

STATIC_URL = "/static/"

SECRET_KEY = "lolimasekrit"

ROOT_URLCONF = "example_project.urls"

# DJANGO1.9 switch to only MIDDLEWARE
MIDDLEWARE_CLASSES = MIDDLEWARE = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
)


INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.admin",
    "example_project.polls",
    # my app
    "django_object_actions",
    # dev helpers
    "django_extensions",
)


TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]



import random
import string

import factory
from django.contrib.auth import get_user_model
from django.utils import timezone

from . import models


class UserFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = get_user_model()

    first_name = factory.Faker("first_name")
    last_name = factory.Faker("last_name")
    username = factory.Faker("slug")
    email = factory.Faker("email")
    password = factory.PostGenerationMethodCall("set_password", "password")


class PollFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = models.Poll

    question = factory.Faker("sentence")
    pub_date = factory.LazyAttribute(lambda __: timezone.now())


class ChoiceFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = models.Choice

    poll = factory.SubFactory(PollFactory)
    choice_text = factory.Faker("word")
    votes = factory.Faker("pyint")


class CommentFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = models.Comment


def get_random_string(length):
    letters = string.ascii_lowercase
    return "".join(random.choice(letters) for i in range(length))


class RelatedDataFactory(factory.django.DjangoModelFactory):
    id = factory.lazy_attribute(
        lambda __: f"{get_random_string(2)}:{get_random_string(2)}-{get_random_string(2)}!{get_random_string(2)}"
    )

    class Meta:
        model = models.RelatedData



from functools import wraps
from itertools import chain

from django.contrib import messages
from django.contrib.admin.utils import unquote
from django.db.models.query import QuerySet
from django.http import Http404, HttpResponseRedirect
from django.http.response import HttpResponseBase, HttpResponseNotAllowed
from django.urls import re_path, reverse
from django.views.generic import View
from django.views.generic.detail import SingleObjectMixin
from django.views.generic.list import MultipleObjectMixin

DEFAULT_METHODS_ALLOWED = ("GET", "POST")
DEFAULT_BUTTON_TYPE = "a"


class BaseDjangoObjectActions:
    """
    ModelAdmin mixin to add new actions just like adding admin actions.

    Attributes
    ----------
    model : django.db.models.Model
        The Django Model these actions work on. This is populated by Django.
    change_actions : list of str
        Write the names of the methods of the model admin that can be used as
        tools in the change view.
    changelist_actions : list of str
        Write the names of the methods of the model admin that can be used as
        tools in the changelist view.
    tools_view_name : str
        The name of the Django Object Actions admin view, including the 'admin'
        namespace. Populated by `_get_action_urls`.
    """

    change_actions = []
    changelist_actions = []
    tools_view_name = None

    # EXISTING ADMIN METHODS MODIFIED
    #################################

    def get_urls(self):
        """Prepend `get_urls` with our own patterns."""
        urls = super().get_urls()
        return self._get_action_urls() + urls

    def change_view(self, request, object_id, form_url="", extra_context=None):
        extra_context = extra_context or {}
        extra_context.update(
            {
                "objectactions": [
                    self._get_tool_dict(action)
                    for action in self.get_change_actions(request, object_id, form_url)
                ],
                "tools_view_name": self.tools_view_name,
            }
        )
        return super().change_view(request, object_id, form_url, extra_context)

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context.update(
            {
                "objectactions": [
                    self._get_tool_dict(action)
                    for action in self.get_changelist_actions(request)
                ],
                "tools_view_name": self.tools_view_name,
            }
        )
        return super().changelist_view(request, extra_context)

    # USER OVERRIDABLE
    ##################

    def get_change_actions(self, request, object_id, form_url):
        """
        Override this to customize what actions get to the change view.

        This takes the same parameters as `change_view`.

        For example, to restrict actions to superusers, you could do:

            class ChoiceAdmin(DjangoObjectActions, admin.ModelAdmin):
                def get_change_actions(self, request, **kwargs):
                    if request.user.is_superuser:
                        return super(ChoiceAdmin, self).get_change_actions(
                            request, **kwargs
                        )
                    return []
        """
        return self.change_actions

    def get_changelist_actions(self, request):
        """
        Override this to customize what actions get to the changelist view.
        """
        return self.changelist_actions

    # INTERNAL METHODS
    ##################

    def _get_action_urls(self):
        """Get the url patterns that route each action to a view."""
        actions = {}

        model_name = self.model._meta.model_name
        # e.g.: polls_poll
        base_url_name = f"{self.model._meta.app_label}_{model_name}"
        # e.g.: polls_poll_actions
        model_actions_url_name = f"{base_url_name}_actions"

        self.tools_view_name = "admin:" + model_actions_url_name

        # WISHLIST use get_change_actions and get_changelist_actions
        # TODO separate change and changelist actions
        for action in chain(self.change_actions, self.changelist_actions):
            actions[action] = getattr(self, action)
        return [
            # change, supports the same pks the admin does
            # https://github.com/django/django/blob/stable/1.10.x/django/contrib/admin/options.py#L555
            re_path(
                r"^(?P<pk>.+)/actions/(?P<tool>\w+)/$",
                self.admin_site.admin_view(  # checks permissions
                    ChangeActionView.as_view(
                        model=self.model,
                        actions=actions,
                        back=f"admin:{base_url_name}_change",
                        current_app=self.admin_site.name,
                    )
                ),
                name=model_actions_url_name,
            ),
            # changelist
            re_path(
                r"^actions/(?P<tool>\w+)/$",
                self.admin_site.admin_view(  # checks permissions
                    ChangeListActionView.as_view(
                        model=self.model,
                        actions=actions,
                        back=f"admin:{base_url_name}_changelist",
                        current_app=self.admin_site.name,
                    )
                ),
                # Dupe name is fine. https://code.djangoproject.com/ticket/14259
                name=model_actions_url_name,
            ),
        ]

    def _get_tool_dict(self, tool_name):
        """Represents the tool as a dict with extra meta."""
        tool = getattr(self, tool_name)
        standard_attrs, custom_attrs = self._get_button_attrs(tool)
        return dict(
            name=tool_name,
            label=getattr(tool, "label", tool_name.replace("_", " ").capitalize()),
            standard_attrs=standard_attrs,
            custom_attrs=custom_attrs,
            button_type=getattr(tool, "button_type", DEFAULT_BUTTON_TYPE),
        )

    def _get_button_attrs(self, tool):
        """
        Get the HTML attributes associated with a tool.

        There are some standard attributes (class and title) that the template
        will always want. Any number of additional attributes can be specified
        and passed on. This is kinda awkward and due for a refactor for
        readability.
        """
        attrs = getattr(tool, "attrs", {})
        # href is not allowed to be set. should an exception be raised instead?
        if "href" in attrs:
            attrs.pop("href")
        # title is not allowed to be set. should an exception be raised instead?
        # `short_description` should be set instead to parallel django admin
        # actions
        if "title" in attrs:
            attrs.pop("title")
        default_attrs = {
            "class": attrs.get("class", ""),
            "title": getattr(tool, "short_description", ""),
        }
        standard_attrs = {}
        custom_attrs = {}
        for k, v in dict(default_attrs, **attrs).items():
            if k in default_attrs:
                standard_attrs[k] = v
            else:
                custom_attrs[k] = v
        return standard_attrs, custom_attrs


class DjangoObjectActions(BaseDjangoObjectActions):
    change_form_template = "django_object_actions/change_form.html"
    change_list_template = "django_object_actions/change_list.html"


class BaseActionView(View):
    """
    The view that runs a change/changelist action callable.

    Attributes
    ----------
    back : str
        The urlpattern name to send users back to. This is set in
        `_get_action_urls` and turned into a url with the `back_url` property.
    model : django.db.model.Model
        The model this tool operates on.
    actions : dict
        A mapping of action names to callables.
    """

    back = None
    model = None
    actions = None
    current_app = None

    @property
    def view_args(self):
        """
        tuple: The argument(s) to send to the action (excluding `request`).

        Change actions are called with `(request, obj)` while changelist
        actions are called with `(request, queryset)`.
        """
        raise NotImplementedError

    @property
    def back_url(self):
        """
        str: The url path the action should send the user back to.

        If an action does not return a http response, we automagically send
        users back to either the change or the changelist page.
        """
        raise NotImplementedError

    def dispatch(self, request, tool, **kwargs):
        # Fix for case if there are special symbols in object pk
        for k, v in self.kwargs.items():
            self.kwargs[k] = unquote(v)

        try:
            view = self.actions[tool]
        except KeyError as exc:
            raise Http404("Action does not exist") from exc

        allowed_methods = getattr(view, "methods", DEFAULT_METHODS_ALLOWED)
        if request.method.upper() not in allowed_methods:
            return HttpResponseNotAllowed(allowed_methods)

        ret = view(request, *self.view_args)
        if isinstance(ret, HttpResponseBase):
            return ret

        return HttpResponseRedirect(self.back_url)

    def message_user(self, request, message):
        """
        Mimic Django admin actions's `message_user`.

        Like the second example:
        https://docs.djangoproject.com/en/1.9/ref/contrib/admin/actions/#custom-admin-action
        """
        messages.info(request, message)


class ChangeActionView(SingleObjectMixin, BaseActionView):
    @property
    def view_args(self):
        return (self.get_object(),)

    @property
    def back_url(self):
        return reverse(
            self.back, args=(self.kwargs["pk"],), current_app=self.current_app
        )


class ChangeListActionView(MultipleObjectMixin, BaseActionView):
    @property
    def view_args(self):
        return (self.get_queryset(),)

    @property
    def back_url(self):
        return reverse(self.back, current_app=self.current_app)


def takes_instance_or_queryset(func):
    """Decorator that makes standard Django admin actions compatible."""

    @wraps(func)
    def decorated_function(self, request, queryset):
        # func follows the prototype documented at:
        # https://docs.djangoproject.com/en/dev/ref/contrib/admin/actions/#writing-action-functions
        if not isinstance(queryset, QuerySet):
            try:
                # Django >=1.8
                queryset = self.get_queryset(request).filter(pk=queryset.pk)
            except AttributeError:
                try:
                    # Django >=1.6,<1.8
                    model = queryset._meta.model
                except AttributeError:  # pragma: no cover
                    # Django <1.6
                    model = queryset._meta.concrete_model
                queryset = model.objects.filter(pk=queryset.pk)
        return func(self, request, queryset)

    return decorated_function


def action(
    function=None,
    *,
    permissions=None,
    description=None,
    label=None,
    attrs=None,
    methods=DEFAULT_METHODS_ALLOWED,
    button_type=DEFAULT_BUTTON_TYPE,
):
    """
    Conveniently add attributes to an action function:

        @action(
            permissions=['publish'],
            description='Mark selected stories as published',
            label='Publish'
        )
        def make_published(self, request, queryset):
            queryset.update(status='p')

    This is equivalent to setting some attributes (with the original, longer
    names) on the function directly:

        def make_published(self, request, queryset):
            queryset.update(status='p')
        make_published.allowed_permissions = ['publish']
        make_published.short_description = 'Mark selected stories as published'
        make_published.label = 'Publish'

    This is the django-object-actions equivalent of
    https://docs.djangoproject.com/en/stable/ref/contrib/admin/actions/#django.contrib.admin.action
    """

    def decorator(func):
        if permissions is not None:
            func.allowed_permissions = permissions
        if description is not None:
            func.short_description = description
        if label is not None:
            func.label = label
        if attrs is not None:
            func.attrs = attrs
        func.methods = methods
        func.button_type = button_type
        return func

    if function is None:
        return decorator
    return decorator(function)



from django.contrib import admin
from django.urls import path

from example_project.polls.admin import support_admin

urlpatterns = [
    path("admin/", admin.site.urls),
    path("support/", support_admin.urls),
]



"""A Django app for adding object tools for models in the admin."""

__version__ = "5.0.0"


from .utils import (
    BaseDjangoObjectActions,
    DjangoObjectActions,
    action,
    takes_instance_or_queryset,
)

__all__ = [
    "BaseDjangoObjectActions",
    "DjangoObjectActions",
    "action",
    "takes_instance_or_queryset",
]



from django.contrib import admin
from django.contrib.admin import AdminSite
from django.db.models import F
from django.http import HttpResponseRedirect
from django.urls import reverse

from django_object_actions import (
    DjangoObjectActions,
    action,
    takes_instance_or_queryset,
)

from .models import Choice, Comment, Poll, RelatedData


class ChoiceAdmin(DjangoObjectActions, admin.ModelAdmin):
    list_display = ("poll", "choice_text", "votes")

    # Actions
    #########

    @action(
        description="+1",
        label="vote++",
        attrs={
            "test": '"foo&bar"',
            "Robert": '"); DROP TABLE Students; ',  # 327
            "class": "addlink",
        },
    )
    @takes_instance_or_queryset
    def increment_vote(self, request, queryset):
        queryset.update(votes=F("votes") + 1)

    actions = ["increment_vote"]

    # Object actions
    ################

    @action(description="-1")
    def decrement_vote(self, request, obj):
        obj.votes -= 1
        obj.save()

    def delete_all(self, request, queryset):
        self.message_user(request, "just kidding!")

    @action(description="0", methods=("POST",), button_type="form")
    def reset_vote(self, request, obj):
        obj.votes = 0
        obj.save()

    def edit_poll(self, request, obj):
        url = reverse("admin:polls_poll_change", args=(obj.poll.pk,))
        return HttpResponseRedirect(url)

    def raise_key_error(self, request, obj):
        raise KeyError

    change_actions = (
        "increment_vote",
        "decrement_vote",
        "reset_vote",
        "edit_poll",
        "raise_key_error",
    )
    changelist_actions = ("delete_all",)


admin.site.register(Choice, ChoiceAdmin)


class ChoiceInline(admin.StackedInline):
    model = Choice
    extra = 3


class PollAdmin(DjangoObjectActions, admin.ModelAdmin):
    # List
    ######

    list_display = ("question", "pub_date", "was_published_recently")
    list_filter = ["pub_date"]
    search_fields = ["question"]
    date_hierarchy = "pub_date"

    def changelist_view(self, request, extra_context=None):
        extra_context = {"foo": "changelist_view"}
        return super().changelist_view(request, extra_context)

    # Detail
    ########

    fieldsets = [
        (None, {"fields": ["question"]}),
        ("Date information", {"fields": ["pub_date"], "classes": ["collapse"]}),
    ]
    inlines = [ChoiceInline]

    def change_view(self, request, object_id, form_url="", extra_context=None):
        extra = {"foo": "change_view"}
        return super().change_view(request, object_id, form_url, extra)

    # Object actions
    ################

    @action(label="Delete All Choices")
    def delete_all_choices(self, request, obj):
        from django.shortcuts import render

        if request.method == "POST":
            obj.choice_set.all().delete()
            return None

        self.message_user(request, "All choices deleted")
        return render(request, "clear_choices.html", {"object": obj})

    def question_mark(self, request, obj):
        """Add a question mark."""
        obj.question = obj.question + "?"
        obj.save()

    change_actions = ("delete_all_choices", "question_mark")

    def get_change_actions(self, request, object_id, form_url):
        actions = super().get_change_actions(request, object_id, form_url)
        actions = list(actions)
        if not request.user.is_superuser:
            return []

        obj = self.model.objects.get(pk=object_id)
        if obj.question.endswith("?"):
            actions.remove("question_mark")

        return actions


admin.site.register(Poll, PollAdmin)


class CommentAdmin(DjangoObjectActions, admin.ModelAdmin):
    # Object actions
    ################

    def hodor(self, request, obj):
        if not obj.comment:
            # bail because we need a comment
            return
        obj.comment = " ".join(["hodor" for x in obj.comment.split()])
        obj.save()

    change_actions = ("hodor",)


admin.site.register(Comment, CommentAdmin)


class RelatedDataAdmin(DjangoObjectActions, admin.ModelAdmin):
    # Object actions
    ################

    def fill_up(self, request, obj):
        if not obj.extra_data:
            # bail because we need a comment
            obj.extra_data = "hodor"
        else:
            obj.extra_data = ""
        obj.save()

    change_actions = ("fill_up",)


admin.site.register(RelatedData, RelatedDataAdmin)


support_admin = AdminSite(name="support")
support_admin.register(Poll, PollAdmin)



"""
Integration tests
"""

from django.urls import reverse

from .tests import LoggedInTestCase


class ChangeListTests(LoggedInTestCase):
    def test_changelist_action_is_rendered(self):
        response = self.client.get(reverse("admin:polls_choice_changelist"))
        self.assertEqual(response.status_code, 200)
        self.assertIn(
            b'href="/admin/polls/choice/actions/delete_all/"', response.content
        )



from unittest import mock

from django.test import TestCase

from example_project.polls.models import Poll

from ..utils import (
    BaseActionView,
    BaseDjangoObjectActions,
    action,
    takes_instance_or_queryset,
)


class BaseDjangoObjectActionsTest(TestCase):
    def setUp(self):
        self.instance = BaseDjangoObjectActions()
        self.instance.model = mock.Mock(
            **{"_meta.app_label": "app", "_meta.model_name": "model"}
        )

    @mock.patch(
        "django_object_actions.utils.BaseDjangoObjectActions.admin_site", create=True
    )
    def test_get_action_urls_trivial_case(self, mock_site):
        urls = self.instance._get_action_urls()

        self.assertEqual(len(urls), 2)
        self.assertEqual(urls[0].name, "app_model_actions")

    def test_get_change_actions_gets_attribute(self):
        # Set up
        self.instance.change_actions = mock.Mock()

        # Test
        returned_value = self.instance.get_change_actions(
            request=mock.Mock(), object_id=mock.Mock(), form_url=mock.Mock()
        )

        # Assert
        self.assertEqual(id(self.instance.change_actions), id(returned_value))

    def test_get_button_attrs_returns_defaults(self):
        # TODO: use `mock`
        mock_tool = type("mock_tool", (object,), {})
        attrs, __ = self.instance._get_button_attrs(mock_tool)
        self.assertEqual(attrs["class"], "")
        self.assertEqual(attrs["title"], "")

    def test_get_button_attrs_disallows_href(self):
        mock_tool = type("mock_tool", (object,), {"attrs": {"href": "hreeeeef"}})
        attrs, __ = self.instance._get_button_attrs(mock_tool)
        self.assertNotIn("href", attrs)

    def test_get_button_attrs_disallows_title(self):
        mock_tool = type(
            "mock_tool",
            (object,),
            {
                "attrs": {"title": "i wanna be a title"},
                "short_description": "real title",
            },
        )
        attrs, __ = self.instance._get_button_attrs(mock_tool)
        self.assertEqual(attrs["title"], "real title")

    def test_get_button_attrs_gets_set(self):
        mock_tool = type(
            "mock_tool",
            (object,),
            {"attrs": {"class": "class"}, "short_description": "description"},
        )
        attrs, __ = self.instance._get_button_attrs(mock_tool)
        self.assertEqual(attrs["class"], "class")
        self.assertEqual(attrs["title"], "description")

    def test_get_button_attrs_custom_attrs_get_partitioned(self):
        mock_tool = type("mock_tool", (object,), {"attrs": {"nonstandard": "wombat"}})
        attrs, custom = self.instance._get_button_attrs(mock_tool)
        self.assertEqual(custom["nonstandard"], "wombat")


class BaseActionViewTests(TestCase):
    def setUp(self):
        super().setUp()
        self.view = BaseActionView()

    @mock.patch("django_object_actions.utils.messages")
    def test_message_user_proxies_messages(self, mock_messages):
        self.view.message_user("request", "message")
        mock_messages.info.assert_called_once_with("request", "message")


class DecoratorTest(TestCase):
    fixtures = ["sample_data"]

    def setUp(self):
        # WISHLIST don't depend on fixture
        self.obj = Poll.objects.get(pk=1)
        self.queryset = Poll.objects.all()

    def test_trivial(self):
        # setup
        def myfunc(foo, bar, queryset):
            return queryset

        # make sure my test function outputs the third arg
        self.assertEqual(myfunc(None, None, "foo"), "foo")
        # or the `queryset` kwarg
        self.assertEqual(myfunc(None, None, queryset="bar"), "bar")

    def test_decorated(self):
        # setup
        @takes_instance_or_queryset
        def myfunc(foo, bar, queryset):
            return queryset

        # passing in an instance yields a queryset (using positional args)
        queryset = myfunc(None, None, self.obj)
        # the resulting queryset only has one item and it's self.obj
        self.assertEqual(queryset.get(), self.obj)

        # passing in a queryset yields the same queryset
        queryset = myfunc(None, None, self.queryset)
        self.assertEqual(queryset, self.queryset)

        # passing in an instance yields a queryset (using keyword args)
        queryset = myfunc(None, None, queryset=self.obj)
        # the resulting queryset only has one item and it's self.obj
        self.assertEqual(queryset.get(), self.obj)


class DecoratorActionTest(TestCase):
    def test_decorated(self):
        # setup
        @action(description="First action of this admin site.")
        def action_1(modeladmin, request, queryset):
            pass

        @action(permissions=["do_action2"])
        def action_2(modeladmin, request, queryset):
            pass

        @action(label="Third action")
        def action_3(modeladmin, request, queryset):
            pass

        @action(
            attrs={
                "class": "addlink",
            }
        )
        def action_4(modeladmin, request, queryset):
            pass

        self.assertEqual(action_1.short_description, "First action of this admin site.")
        self.assertEqual(action_2.allowed_permissions, ["do_action2"])
        self.assertEqual(action_3.label, "Third action")
        self.assertEqual(
            action_4.attrs,
            {
                "class": "addlink",
            },
        )



# Generated by Django 1.9.2 on 2016-02-25 17:25

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Choice",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("choice_text", models.CharField(max_length=200)),
                ("votes", models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name="Comment",
            fields=[
                (
                    "uuid",
                    models.UUIDField(editable=False, primary_key=True, serialize=False),
                ),
                ("comment", models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name="Poll",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("question", models.CharField(max_length=200)),
                ("pub_date", models.DateTimeField(verbose_name=b"date published")),
            ],
        ),
        migrations.AddField(
            model_name="choice",
            name="poll",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="polls.Poll"
            ),
        ),
    ]
